import time
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score
from sklearn.base import clone
from sklearn.tree import DecisionTreeClassifier
import tensorflow as tf
from animal_shape_mnist import dataset_Loader

def benchmark_model(model, X, y, model_name, dataset_name, repeats=5, is_tf_model=False):
    results = []
    for _ in range(repeats):
        X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)
        
        if not is_tf_model and X_train.ndim == 3:
            X_train = X_train.reshape(X_train.shape[0], -1)
            X_test = X_test.reshape(X_test.shape[0], -1)

        start_time = time.time()
        if is_tf_model:
            X_train = X_train.reshape(-1, 28, 28, 1).astype("float32") / 255.0
            X_test = X_test.reshape(-1, 28, 28, 1).astype("float32") / 255.0
            model.fit(X_train, y_train, epochs=1, batch_size=32, verbose=0)
            y_pred = model.predict(X_test).argmax(axis=1)
        else:
            model.fit(X_train, y_train)
            y_pred = model.predict(X_test)

        train_time = time.time() - start_time
        accuracy = accuracy_score(y_test, y_pred)
        results.append((accuracy, train_time))

    accuracies = [r[0] for r in results]
    times = [r[1] for r in results]

    return {
        "Model": model_name,
        "Dataset": dataset_name,
        "Accuracy (mean)": np.mean(accuracies),
        "Accuracy (std)": np.std(accuracies),
        "Training Time (mean)": np.mean(times),
    }

def run_benchmark(models, repeats=5):
    """Runs benchmarks on models using datasets from dataset_Loader."""
    datasets = {
        "MNIST": dataset_Loader.load_mnist_data(),
        "Fashion-MNIST": dataset_Loader.load_fashion_mnist_data(),
        "Animal Shape-MNIST Dataset": dataset_Loader.load_animal_shape_dataset()
    }

    benchmark_results = []
    for model_name, model in models.items():
        is_tf_model = isinstance(model, tf.keras.Model)
        for dataset_name, (X, y) in datasets.items():
            print(f"Running benchmark for {model_name} on {dataset_name}...")
            result = benchmark_model(model, X, y, model_name, dataset_name, repeats, is_tf_model)
            benchmark_results.append(result)

    return pd.DataFrame(benchmark_results)
